"""
A presentation about Python!
"""
import code
import os
import sys

class PresentConsole(code.InteractiveConsole):
    """
    A console that models the Python interpreter except...

    Empty newlines execute the next block of python code...
    """
    def __init__(self, prespath, slidenum=0, *args, **kwargs):
        self._prespath = prespath
        self._slidenum = slidenum
        code.InteractiveConsole.__init__(self, *args, **kwargs)
        with open(prespath, 'r') as presfile:
            self.slides = presfile.read().split("#--newslide")

    def statusing(self):
        """
        Print the status of our presentation
        """
        status = "Slide {0} of {1}".format(self._slidenum, len(self.slides))
        print(status)
        return

    def codeslide(self, slide):
        """
        Runs a slide with some code in it.

        Arguments:
        - `slide`: str
        """
        stripped = slide[7:].strip()
        for x in stripped.split("\n"):
            print(sys.ps1 + x)
            super(PresentConsole, self).push(x)
        return

    def simpleslide(self, slide):
        """
        This is text rite? I guess we'll just print() it

        Arguments:
        - `slide`: str
        """
        print(slide)
        return

    def splitslide(self, slide):
        """
        This slide wants to get printed bit by bit.

        Let's do that.

        Arguments:
        - `slide`:
        """
        slide = slide.strip()
        sections = slide[8:].split('#----')
        for i, section in enumerate(sections):
            if section == '':
                continue
            print(section.strip())
            if i + 1 < len(sections):
                input = self.raw_input()
                if input != '':
                    raise ValueError("Entered code in a splitside. Stop it.")
        return

    def prevslide(self):
        """
        Go back one slide!
        """
        self._slidenum -= 2
        return self.runslide("")

    def runslide(self, line):
        """
        Print then run a slide
        """
        if self._slidenum >= len(self.slides):
            return
        os.system('clear')
        if line.strip() == "^":
            return self.prevslide()
        else:
            slide =  self.slides[self._slidenum].strip()
            self._slidenum += 1
        self.statusing()
        if slide.startswith("#--code"):
            return self.codeslide(slide)
        if slide.startswith('#--split'):
            return self.splitslide(slide)
        return self.simpleslide(slide)

    def push(self, line):
        """
        If we're passed an empty string, get the next part of the presentation
        """
        if line == "" or line == "^":
            return self.runslide(line)
        return super(PresentConsole, self).push(line)



"""
Let's make this a package!
"""
import argparse
import sys

from pypres.pres import PresentConsole

def main():
    """
    OH: You want to run a program then?
    """
    parser = argparse.ArgumentParser(
        description = "Presenting"
        )
    parser.add_argument("presentation",  help = "Presentation file to run")
    parser.add_argument("-s", "--slide", help = "Start at slide num (0 indexed)", default = 0)
    args =  parser.parse_args()

    sys.ps1 = '{0}.{1} >>> '.format( *sys.version_info)
    replish = PresentConsole(args.presentation, slidenum=int(args.slide))
    replish.interact()




import re

from distutils.core import setup

VERSION_FILE = "pypres/_version.py"
verstrline = open(VERSION_FILE, "rt").read()
VSRE = r'^__version__ = [\'"]([^\'"]*)[\'"]$'
mo = re.search(VSRE,  verstrline, re.M)
if mo:
    VERSION = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in {0}".format(VERSION_FILE))

LONG_DESC = """
Make Python related presentations in a Python Repl!
"""
setup(
    name = "pypres",
    version = VERSION,
    author = "David Miller",
    author_email = "david@deadpansincerity.com",
    url = "https://github.com/davidmiller/pypres",
    description = "Presentation in a Python REPL",
    long_description = LONG_DESC,
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Topic :: Communications",
        "Topic :: Education",
        "Topic :: Office/Business",
        "Topic :: Utilities"
        ],
    packages = ['pypres'],
    install_requires = []
    )
